/**
 * Klasse som holder orden p treningsettet, hvilke subject som hrer til i treningsett og
 * vektorer for disse.
 * @author Toril
 * @version mai 06
 */
import java.util.*;
import java.io.*;
public class RelevantSubjects{
	private HashSet identifiedSubject;
	private ArrayList orderedIdfSubject;
	private ArrayList meanVectors;
	/**
	 * Konstruktr som oppretter et HashSet og en ArrayList over identifiserte subject for treningsettet.
	 * Samt en ArrayList med gjennomsnittsvektorer for hvert subject.
	 */
	public RelevantSubjects(){
		this.identifiedSubject = new HashSet();
		this.orderedIdfSubject = new ArrayList();
		this.meanVectors = new ArrayList();
	}
	/**
	 * Metode som legger identifisert Subject til i listen over identifiserte Subject, dersom det ikke ligger
	 * i listen fra fr
	 * @param subject Subject
	 */
	public void addSubject(Subject subject){
		if(!identifiedSubject.contains(subject) && !orderedIdfSubject.contains(subject)){
			identifiedSubject.add(subject);
			orderedIdfSubject.add(subject);
		}
	}
	/**
	 * Metode som sjekker om et Subject befinner seg i listen over identifiserte Subject, ved  sende inn
	 * navnet p Subjectet som en streng
	 * @param etSubject String
	 * @return boolean
	 */
	public boolean isInList(String etSubject){
		boolean found = false;
		if(orderedIdfSubject.size() != 0){
			for(int i=0; i<orderedIdfSubject.size(); i++){
				String en = orderedIdfSubject.get(i).toString();
				String to = etSubject;
				if(en.equals(to)){
					found = true;
				}
			}
		}
		return found;
	}
	/**
	 * Metode som finne index av et Subject i ArrayListen over identifiserte Subjects, ved  sende inn navnet
	 * p Subject som en String
	 * @param etSubject String
	 * @return index int
	 */
	public int indexOfString(String etSubject){
		int index = -1;
		for(int i=0; i<orderedIdfSubject.size(); i++){
			if(orderedIdfSubject.get(i).toString().equals(etSubject)){
				index = i;
			}
		}
		return index;
	}
	/**
	 * Metode som setter gjennomsnittsvektorer for Subjects etter at alle dokumenter i treningsettet er 
	 * lagt til Det er noe ulogisk  sende med VektorModell her, men det er gjort fordi man trenger tilgang 
	 * til samlingen for  f gjennomfrt utregningen.
	 * @param enModell VektorModell
	 */
	public void setMeanVectors(VektorModell enModell){
		Samling collection = enModell.getCollection();
		for(int i=0; i<orderedIdfSubject.size(); i++){
			Subject subject = (Subject)orderedIdfSubject.get(i);
			subject.setNormalizedMeanVector(collection);
			double[] meanVector = subject.getNormMeanVector();
			meanVectors.add(meanVector);
		}
	}
	/**
	 * Metode som returnerer en liste over alle gjennomsnittsvektorene for treningssettet.
	 * @return meanVectors ArrayList
	 */
	public ArrayList getMeanVectorList(){
		return meanVectors;
	}
	/**
	 * Metode som returnerer de fem likeste subjectene til et dokument i form av en ArrayList.
	 * Ogs her virker det ulogisk  sende med vektorModell, men det er gjort fordi en vektormodell i utgangspunktet
	 * skal representere et dokument.
	 * @param enModell VektorModell
	 * @return femUtvalgte ArrayList
	 */
	public ArrayList returnProbbleSubject(VektorModell enModell){
		double[] docVectorNorm = enModell.getDocument().getNormVector();
		double[] subVectorNorm;
		HashMap alleMap = new HashMap();
		ArrayList femUtvalgte = new ArrayList(); 
		Double skalarprod;
		for(int i=0; i<getSize(); i++){
			Subject subject = (Subject)getOrderedSubject().get(i);
			subVectorNorm = (double[])meanVectors.get(i);
			double skalar = enModell.skalarProduktVektor(docVectorNorm, subVectorNorm);
			skalarprod = new Double(skalar);
			alleMap.put(subject, skalarprod);
		}
		Collection coll = alleMap.values();
		Set set = alleMap.keySet();
		Iterator it = set.iterator();
		while(it.hasNext()){
			skalarprod = (Double)Collections.max(coll);
			Subject key = (Subject)it.next();
			Double value = (Double)alleMap.get(key);
			if(value.doubleValue()==skalarprod.doubleValue()){
				coll.remove(skalarprod);
				set = alleMap.keySet();
				it = set.iterator();
				if(femUtvalgte.size()<5){
					femUtvalgte.add(key);
				}
			}
		}
		//skrivProbableSubjectTilFil(enModell, femUtvalgte);
		return femUtvalgte;
	}
	/**
	 * Metode som tmmer listene i Subjectene i treningsettet for dokumenter. Dette blir gjort nr alle sammenligninger
	 * med treningsettet er ferdig.
	 *
	 */
	public void emptyDocTablesInSubjects(){
		for(int i=0; i<orderedIdfSubject.size(); i++){
			Subject subject = (Subject)orderedIdfSubject.get(i);
			subject.emptyDocumentsTables();
		}
	}
	/**
	 * Metode som returnerer str p ArrayListen av Subject
	 * @return size int
	 */
	public int getSize(){
		return orderedIdfSubject.size();
	}
	/**
	 * Metode som gjr ArrayListen med subject om til en String
	 * @return String
	 */
	public String toString(){
		return orderedIdfSubject.toString();
	}
	/**
	 * Metod som returnerer ArrayList med Subject
	 * @return orderedIdfSubject ArrayList
	 */
	public ArrayList getOrderedSubject(){
		return orderedIdfSubject;
	}
	/**
	 * Metode som returnerer HashSet med Subject
	 * @return identifiedSubject HashSet
	 */
	public HashSet getSubjectSet(){
		return identifiedSubject;
	}
}
