/** Klasse Samling, holder orden p dokumentene i samlingen til E-lis der sprk er angitt til Engelsk
 * @author Toril
 * @version mai 06
 */
import java.util.*;
public class Samling {
	private ArrayList collection;
	private Vocabulary vocabulary;
	private DbId idListe;

	/** Konstruktr som oppretter en Arraylist collection for alle dokumentene og Vocabulary og DbId 
	 *
	 */
	public Samling(){
		collection = new ArrayList();
		vocabulary = new Vocabulary();
		idListe = new DbId();
	}
	/** Metode som legger til document i ArrayListen til samlingen og i ordlisten.
	 * Nb! Metoden brukes bare nr man ikke har en fil med ordliste fra fr.
	 * @param etDocument Document
	 */
	public void addDocument(Document etDocument){
		collection.add(etDocument);
		vocabulary.addDocument(etDocument);
	}
	/** Metode som henter Vocabulary og gjr den tilgjengelig for andre.
	 * 
	 * @return vocabulary Vocabulary
	 */
	public Vocabulary getVocabulary(){
		return vocabulary;
	}
	/** Metode som henter ArrayList med samling
	 * 
	 * @return collection ArrayList
	 */
	public ArrayList getCollection(){
		return collection;
	}
	/** Metode som setter samlingen.
	 * 
	 * @param collection ArrayList
	 */
	public void setCollection(ArrayList collection) {
		this.collection = collection;
	}
	/** Metode som setter Vocabulary
	 * 
	 * @param vocabulary Vocabulary
	 */
	public void setVocabulary(Vocabulary vocabulary) {
		this.vocabulary = vocabulary;
	}
	/** Metode som henter ArrayList med Vocabulary
	 * 
	 * @return orderedVocabulary ArrayList
	 */
	public ArrayList getTheVocabulary(){
		return vocabulary.getOrderedVocabulary();
	}
	/**Metode som lager normaliserte vektorer av dokumentene, bruker metode i Document.
	 * 
	 *
	 */
	public void lagNormalizedVectors(){
		for(int i=0; i<collection.size(); i++){
			Document doc = (Document)collection.get(i);
			doc.setNormVector(this);
		}
	}
	/** Metode som legger til dokument til samling uten  legge til i vocabulary.
	 * Legger og til Subjecter i listen over reelle subject til hvert Document.
	 *
	 */
	public void addDocumentToDocumentList(){
		ArrayList id = idListe.getId();
		if(id.isEmpty()){
			idListe.setIdTabell();
		}
		Subject sub;
		for(int i=0; i<getAntallDoc(); i++){//idListe.getTableSize()
			System.out.println(i);
			String title = idListe.getKolInn(i, "title");
			String description = idListe.getKolInn(i, "description");
			Document doc = new Document(title, description);
			if(!collection.contains(doc)){
				//addDocument(doc); //Bare nr ordlisten skal lages
				collection.add(doc);
				String wholeSubject = idListe.getKolInn(i, "subject");
				StringTokenizer token = new StringTokenizer(wholeSubject, "|");
				while(token.hasMoreTokens()){
					String subjectName = token.nextToken().trim();
					subjectName = fjernTegnSubject(subjectName).trim();
					SubjectList sublist = new SubjectList();
					if(sublist.isInList(subjectName)){
						int index = sublist.getSubjectIndex(subjectName);
						if(index >= 0){
							sub = (Subject)sublist.getSubjectList().get(index);
							doc.addSubject(sub);
						}
					}
				}
			}
		}
		lagNormalizedVectors();
	}
	/** Oppretter et RelevantSubjects objekt for angitt del av samling.
	 * 
	 * @param start int startindex for samling.
	 * @param stop int sluttindex for samling
	 * @return relevants RelevantSubjects
	 */
	public RelevantSubjects getNextDocument(int start, int stop){
		RelevantSubjects relevants = new RelevantSubjects();
		for(int i=start; i<stop; i++){
			Document doc = (Document)collection.get(i);
			for(int j=0; j<doc.getSubjectThisDocument().size(); j++){
				Subject subject = (Subject)doc.getSubjectThisDocument().get(j);
				subject.addDocument(doc);
				relevants.addSubject(subject);
			}
			
		}
		VektorModell enVektor = new VektorModell((Document)collection.get(0), this);
		relevants.setMeanVectors(enVektor);
		return relevants;
	}
	/** Metode som finner antall dokumenter i hele samlingen ved  kontakte DbKobling klassen og kjre count kommando i SQL
	 * 
	 * @return antallDoc int
	 */
	public int getAntallDoc(){
		DbKobling kobling = new DbKobling();
		return kobling.getTablsize();
	}
	/** Metode som fjerner undvendige tegn i Subject Stringen.
	 * 
	 * @param subject String
	 * @return subjectSring String
	 */
	public String fjernTegnSubject(String subject){
		for(int i = 0; i<subject.length(); i++){
			char a = subject.charAt(i);
			if(a == ',' || a == '.' || a== '\"'|| a== '(' || a==')' || a == '!' || a == '?' || a ==':' || a== ';' || a=='[' ||a==']'){
					char b = ' ';
					subject = subject.replace(a, b);
			}
		}return subject;
	}
}
