/** VektorModell er den viktigste klassen i systemet.
 * @author Toril
 * @version mai 06
 * 
 */
import java.util.*;
public class VektorModell {
		private HashMap termHashMap;
		private int ordAntall;
		private Samling collection;
		private Document document;
		
	
	/** Konstruktr som tar inn Document objekt og Samling objekt.
	 * Initierer dokumentet nr en vektormodell blir opprettet, oppretter HashMap for dokumentet.
	 * @param etdokument Document
	 * @param collection Samling
	 */
	public VektorModell(Document etdokument, Samling collection){
		termHashMap = new HashMap();
		ordAntall = 0;
		this.collection = collection;
		initiateDocument(etdokument);
		this.document = etdokument;
	}
	
	/** Konstruktr som tar inn et Subject objekt, brukes nr metoden meanVektor(Subject subject) kalles.
	 * @param collection Samling
	 * @param subject Subject
	 */
	public VektorModell(Samling collection, Subject subject){
		this.collection = collection;
	}
	/** Metoden henter et dokument.
	 * @return document Document
	 */
	public Document getDocument(){
		return document;
	}
	/** Metode som legger til et ord i Hashmapen og ker verdien av ordet dersom det finnes fra fr i HashMapen.
	 * @param ord String
	 */
	private void leggTilOrd(String ord){
		ord = ord.trim();
		ordAntall++;
		if(termHashMap.containsKey(ord)){
			Integer i = (Integer) termHashMap.get(ord);
			Integer j = new Integer (i.intValue()+ 1);
			termHashMap.put(ord, j);
		}
		else{
			termHashMap.put(ord, new Integer(1));
		}
	}
	/** Metoden initierer et dokument, ved at et dokument sendes inn nr metoden kalles og ordene i dokumentet blir lagt i HashMapen.
	 * @param etDokument Document
	 */
	private void initiateDocument(Document etDokument){
		ArrayList document = etDokument.getTerms();
		for(int i = 0; i<document.size(); i++){
			leggTilOrd((String)document.get(i));
		}				
	}
	/** Metode som finner antall ord i HashMap.
	 * @return ordAntall int
	 */
	public int hentAntallOrd(){
		return ordAntall;
	}
	/** Metode som finner antallet av et ord i HashMap
	 * @param ord String
	 * @return antall ord int
	 * 
	 */
	public int hentAntallEtOrd(String ord){
		Integer antall = (Integer)termHashMap.get(ord);
		return antall.intValue();
		
	}
	/** Henter antall nkler i HashMap
	 * @return antall nkler i HashMap int
	 * 
	 */
	public int hentAntallKeys(){
		return termHashMap.size();
	}
	/** Metode som henter ut alle ordene (nklene i hasmapen)og returnerer dem som strenger.
	 * @return set Set
	 * 
	 */	
	public Set hentOrdIHashMap(){
		Set set = termHashMap.keySet();
		return set;
	}
	/** Metode som lager vektor av et dokument, ved  la hvert ord i dokumentet som samsvarer med
	 * ord i dokument ker med en, resten av ordene fr 0 som verdi.
	 * @return vector double[]
	 * 
	 */
	public double[] lagVector(){
		int size = collection.getVocabulary().getOrderedVocabulary().size();
		double[] vector = new double[size];
		Set set = this.hentOrdIHashMap();
		Iterator it = set.iterator();
		while(it.hasNext()){
			String key = (String)it.next();
			if(collection.getVocabulary().getVocabulary().contains(key)){
				int i = collection.getVocabulary().getOrderedVocabulary().indexOf(key);
				int j = this.hentAntallEtOrd(key);
				vector[i] = j;
			}
		}
		return vector;
	}
	/** Metode som lager en tabell av double[], som er en normalisert utgave av vektor for dokument.
	 * @param otherModel VektorModell
	 * @return normalizedVector double[] 
	 */
	public double[] lagNormalizedVector(VektorModell otherModel){
		int size = collection.getVocabulary().getOrderedVocabulary().size();
		double [] normalizedVector = new double[size];
		Set set = otherModel.hentOrdIHashMap();
		Iterator it = set.iterator();
		while(it.hasNext()){
			String key = (String)it.next();
			int i = collection.getVocabulary().getOrderedVocabulary().indexOf(key);
			int j = otherModel.hentAntallEtOrd(key);
			normalizedVector[i] = j;
		}
		double absVektor = absVektor(normalizedVector);
		for(int i=0; i<size; i++){
			normalizedVector[i] = normalizedVector[i]/absVektor;
		}
		return normalizedVector;
	}
	/** Metode som legger sammen alle tallene i en double[] tabell.
	 * @param a double[]
	 * @return sum double
	 * 
	 */
	public double leggSammen(double[] a){
		double sum=0;
		for(int i = 0; i<a.length; i++){
				sum = sum + a[i];
		}return sum;
		
	}
	/** Metode som legger sammen to double[] vektorer.
	 * @param a double[]
	 * @param b double[]
	 * @return sum double[]
	 * 
	 */
	public double[] leggsammenTovektorer(double[] a, double[] b){
		double[] sum = new double[a.length];
		if(a.length == b.length){
			for(int i=0; i<a.length; i++){
				sum[i] = a[i] + b[i];
			}
		}return sum;
		
	}
	
	/** Metode som finner gjennomsnittsvektor for et Subject objekt som har en liste over dokumnenter under seg
	 * @param subject Subject
	 * @return meanVektor double[]
	 */
	public double[] meanVektor(Subject subject){
		int vektorSize = collection.getVocabulary().getOrderedVocabulary().size();
		int antallVektorer = subject.getAllDocuments().size();
		double[] meanVektor = new double[vektorSize];
		ArrayList docVektor = new ArrayList();
		double[] d;
		for(int i=0; i<antallVektorer; i++){
			Document doc = (Document)subject.getAllDocuments().get(i);
			VektorModell docModell = new VektorModell(doc, collection);
			double[] docuVector =docModell.lagVector();
			docVektor.add(docuVector);
			d = (double[])docVektor.get(i);
			meanVektor = leggsammenTovektorer(meanVektor, d);
		}
		for(int j=0; j<vektorSize; j++){
			meanVektor[j] = meanVektor[j]/antallVektorer;
		}
		return meanVektor;		
	}
	/** Metode som tar inn to vektorer og finner skalarproduktet til de to vektorene. Vektorene m vre av samme
	 * dimensjon. 
	 * @param normDocVector double[]
	 * @param normMeanSubject double[]
	 * @return skalarprodukt double
	 * 
	 */
	public double skalarProduktVektor(double[] normDocVector, double[] normMeanSubject){
		double[] produkt = new double[normDocVector.length];
		for(int i=0; i<normDocVector.length; i++){
				produkt[i] = normDocVector[i] * normMeanSubject[i];
		}return leggSammen(produkt);
	}
	/**Metode som finner absolutt verden til en vektor, tar inn en vektor i form av en double[].
	 * @param vector double[]
	 * @return absVektor double
	 */
	public double absVektor(double[] vector){
		double absolutt=0;
			for(int i=0; i<vector.length; i++){
				absolutt = absolutt + Math.pow(vector[i], 2)  ;
			}
		return Math.sqrt(absolutt);
	}
	/** Metode som returnerer et Samlings objekt.
	 * @return collection Samling
	 * 
	 */
	public Samling getCollection(){
		return collection;
	}
	/** Metode som returnerer indexen til et ord som blir sendt inn til metoden.
	 * @param ord String 
	 * @return index int
	 * 
	 */
	public int hentIndexAvOrd(String ord){
		int index = collection.getTheVocabulary().indexOf(ord);
		return index;
	}
	
}

	
	

