/*
 * @(#)ConnectionServlet
 *
 * Copyright (c) 1998 Karl Moss. All Rights Reserved.
 *
 * You may study, use, modify, and distribute this software for any
 * purpose provided that this copyright notice appears in all copies.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @version 1.0
 * @date    20Mar99
 *
 */

package db;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
/**
 * <p>This is a simple servlet that holds a global connection
 * pool. The Servlet context is used to store a named attribute
 * (this servlet) so that other servlets have access to the
 * connection pool
 */

public class ConnectionServlet extends HttpServlet
{
  // Our connection pool.
  ConnectionPool m_connectionPool;

  // Context attribute key
  public static String KEY = "ConnectionServlet";


  /**
    * <p>Initialize the servlet. This is called once when the
    * servlet is loaded. It is guaranteed to complete before any
    * requests are made to the servlet
    *
    * @param cfg Servlet configuration information
    */
  public void init(ServletConfig cfg)
    throws ServletException
    {
      super.init(cfg);

      // Create our connection pool
      m_connectionPool = new ConnectionPool();

      // Initialize the connection pool. This will start all
      // of the connections as specified in the connection
      // pool configuration file
      try {
        m_connectionPool.initialize();
	m_connectionPool.setInuseTimeout();
      }
      catch (Exception ex) {
        // Convert the exception
        ex.printStackTrace();
        throw new ServletException
          ("Unable to initialize connection pool");
      }

      // Add this servlet to the context so that other servlets
      // can find us
      getServletContext().setAttribute(KEY, m_connectionPool);
      System.out.println("db init is done");
    }
 public void doGet (HttpServletRequest request,
                       HttpServletResponse response)
        throws ServletException, IOException
    {
	 response.setContentType("text/html");
        PrintWriter out = response.getWriter();
	out.println(m_connectionPool.printPool());
	out.close();
}  
  /**
    * <p>Destroy the servlet. This is called once when the servlet
    * is unloaded.
    */

  public void destroy()
    {
      // Remove the attribute from the context
      getServletContext().removeAttribute(KEY);

      // Tear down our connection pool if it was created
      if (m_connectionPool != null) {
        m_connectionPool.destroy();
      }
      super.destroy();
    }

}

