package ns;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.text.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;


import db.*;
/**
 * This class fetches data provider list from database.
 *
 * database schema: create table dplist(identifier varchar(20) primary key, baseurl varchar(100), active char(1),
 * harvestmethod varchar(20));
 * Used by Harvesting Tools.
 *
 * Author xiaoming Liu (liu_x@cs.odu.edu)
 */

/**
 *  Add one field for table dplist, sets varchar(20);
 */

public class DBDPReader implements NameList
{
    private Vector list;
    ConnectionPool m_connectionPool=null;
    /**
     * Default Constructor --
     * Construct a new data provider reader that inherits Hashtable class
     *
     */
    public DBDPReader(ConnectionPool cpool)
    {
         list=new Vector();
         m_connectionPool=cpool;
    }


    /**
     * Parsing the specified XML document containing the information of data providers
     *
     * @param xmlUri the uri of XML document
     * @return None
     */
    public void init() {
        Statement stmt=null;

        Connection connection=null;

        try{

            connection=m_connectionPool.getConnection();
            stmt=connection.createStatement();
            ResultSet rs=stmt.executeQuery("select identifier, baseURL, sets, lastharvesttime,active, harvestmethod, metaFormat, oaiversion from dplist");
            while(rs.next()){
                DPName dp=new DPName();
                dp.id = rs.getString("identifier");
                dp.baseURL = rs.getString("baseURL");
		dp.sets = rs.getString("sets");
                
                dp.metaFormat = rs.getString("metaFormat");

		if (rs.getDate("lastharvesttime")!=null){
		    SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss");

 
		    dp.lastharvesttime = formatter.format(rs.getDate("lastharvesttime"));
		}
                if ((rs.getString("active")==null)||(rs.getString("active").equals("N")))
                    dp.active=false;
                else
                    dp.active=true;

                if (rs.getString("harvestmethod")==null)
                    dp.harvestMethod="history_mass";
                else
                    dp.harvestMethod = rs.getString("harvestmethod");

                if (rs.getString("oaiversion")==null)
                    dp.oaiversion="1.1";
                else
                    dp.oaiversion = rs.getString("oaiversion");

                //list.put(dp.id,dp);
		list.add(dp);
            }


        }catch(Exception e){
            System.out.println(e);
        }
       finally{
            try{
                stmt.close();
                m_connectionPool.close(connection);
            }
            catch(Exception e){
                System.out.println(e);
            }
        }

    }

    /**
     * Return parse results
     */

     public Vector getNameList(){
        return list;
     }


    /**
     * This main function demonstrates class DPReader
     * @param args the URI of the XML document
     * @return None
     */
    public static void main(String args[]) throws java.lang.Exception
    {

        ConnectionPool cpool=new ConnectionPool();
        cpool.initialize ();
    	DBDPReader dpr = new DBDPReader(cpool);
        dpr.init();
	    for (Enumeration keys = dpr.getNameList().elements(); keys.hasMoreElements();)
    	    System.out.println(keys.nextElement());

    }
}










