package ns;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.text.*;

/**
 * This class parses the XML document containing the information of Data Providers.
 * Used by Harvesting Tools.
 *
 * @author Dun Tan (tan_d@cs.odu.edu)
 * Modified by xiaoming Liu
 */

public class XMLDPReader implements NameList
{
    // private Hashtable list;
    private Vector list;
    String xmlurl;
    /**
     * Default Constructor --
     * Construct a new data provider reader that inherits Hashtable class
     *
     */
    public XMLDPReader()
    {
        this("./conf/dps.xml");
    }
    public XMLDPReader(String url){
        xmlurl=url;
    }


    /**
     * Parsing the specified XML document containing the information of data providers
     *
     * @return None
     */
    public void init() {
	Document doc = null;
	//list=new Hashtable();
         list=new Vector();
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();

         ClassLoader cl = getClass().getClassLoader();
             // Attempt to open an input stream to the configuration file.
          // The configuration file is considered to be a system
          // resource.
          java.io.InputStream in;

          if (cl != null) {
            in = cl.getResourceAsStream(xmlurl);
          }
          else {
            in = ClassLoader.getSystemResourceAsStream(xmlurl);
          }

          // If the input stream is null, then the configuration file
          // was not found
          if (in == null) {
            throw new Exception("Data Provider configuration file '" +
                           xmlurl + "' not found");
              }
    	    doc = builder.parse(in);
	    } catch (Exception err) {
	    System.err.println("An error occurred: " + err);
	}

	// Now, we have the DOM tree
	if (doc != null) {
	    String root = doc.getDocumentElement().getTagName();

	    // Accessing the information needed
	    if (root != null) {
		//System.out.println(root);

		for (Node node = doc.getDocumentElement().getFirstChild();
		     node != null; node = node.getNextSibling()) {

		    //System.out.println(node.getNodeName());

		    if (node.getNodeName().equals("DP")) {
			//System.out.println("Processing DPs");
                        DPName name=new DPName();

			for (Node dp = node.getFirstChild();
			     dp != null; dp = dp.getNextSibling()) {
			    //System.out.println(dp.getNodeName());
			    if (dp.getNodeName().equals("ID")) {
                               if (dp.getFirstChild()!=null)
				    name.id = dp.getFirstChild().getNodeValue();
				//System.out.println("key = " + key);
			     }

			    if (dp.getNodeName().equals("SETS")) {
                    if (dp.getFirstChild()!=null)
				    name.sets  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }

			     if (dp.getNodeName().equals("LASTHARVESTTIME")) {
				if (dp.getFirstChild()!=null)
				    name.lastharvesttime  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }


			    if (dp.getNodeName().equals("HARVESTURL")) {
                    if (dp.getFirstChild()!=null)
				    name.baseURL  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }


  			    if (dp.getNodeName().equals("ACTIVE")) {
                    if (dp.getFirstChild()!=null)
				    if(dp.getFirstChild().getNodeValue().equals("yes"))
                        name.active=true;
                    else
                        name.active=false;
				//System.out.println("value = " + value);
			    }

    			    if (dp.getNodeName().equals("HARVESTMETHOD")) {
                                if (dp.getFirstChild()!=null)
				    name.harvestMethod  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }

	            if (dp.getNodeName().equals("METAFORMAT")) {
                         if (dp.getFirstChild()!=null)
				    name.metaFormat  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }

		     
                    if (dp.getNodeName().equals("OAIVERSION")) {
                         if (dp.getFirstChild()!=null)
				    name.oaiversion  = dp.getFirstChild().getNodeValue();
				//System.out.println("value = " + value);
			    }

			}
			// Save the parsed information
			//list.put(name.id, name);
			list.add(name);
		    }
		}
	    }
	    else
		System.out.println("root is null");
	}
	else
	    System.out.println("doc is null");
    }

    /**
     * Return parse results
     */

     public Vector getNameList(){
        return list;
     }


    /**
     * This main function demonstrates class DPReader
     * @param args the URI of the XML document
     * @return None
     */
    public static void main(String args[])
    {

	XMLDPReader dpr = new XMLDPReader();
	System.out.println("Instantiate");

	//for (Enumeration keys = dpr.getNameList().keys(); keys.hasMoreElements();)
	//    System.out.println(dpr.getNameList().get(keys.nextElement()));
        
         for (Enumeration keys = dpr.getNameList().elements(); keys.hasMoreElements();)
	    System.out.println(keys.nextElement());

    }
}










