package scheduler;


import java.sql.*;
import java.io.*;

import java.net.*;
import java.util.*;
import dienst.*;
import java.text.*;
import db.*;


public abstract  class Record2DB{
    ConnectionPool m_connectionPool=null;
    Connection connection=null;
    PreparedStatement freestmt=null,dcstmt=null, mainstmt=null;

    public Record2DB(ConnectionPool pool){
        m_connectionPool=pool;

    }

    public void initstmt(){
        try{
        connection=m_connectionPool.getConnection();
        freestmt = connection.prepareStatement("insert into free(id,archive,alltext,groupdate,datestamp) values (?,?,?,?,?)");
        dcstmt = connection.prepareStatement("insert into dc (id,archive,datestamp,sets,title,creator,subject,description,publisher,contributor,discovery,type,format,identifier,source,language,relation,coverage,rights,groupdate,cache) values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");

        mainstmt = connection.prepareStatement("insert into main (id, format, archive, sets, metadata, timestamp) values (?, ?, ?, ?, ?, ?)");

        }catch(Exception e){
            System.out.println(e);
        }

   }

   public void closeConnection(){

       try{
	   freestmt.close();
	   dcstmt.close(); 
	   mainstmt.close();
       }
       catch(Exception e){
	   System.out.println(e);
       }

       if (connection!=null)
            m_connectionPool.close(connection);
	
        connection=null;
   }

    /** Added by Yang Zhao, Mar 2002 */
    private int mainUpdate(dienst.Metadata metadata) throws SQLException{
       
       mainstmt.setString(1,clean(metadata.m_identifier));
       mainstmt.setString(2,clean(metadata.m_format));
      
       mainstmt.setString(3,clean(metadata.m_archive));
       mainstmt.setString(4,clean(metadata.m_sets));
        mainstmt.setString(6,clean(metadata.m_timestamp));
       try {
	   
	   //setString() doesn't work when the string is long. setCharacterStream() is used instead. 
	   StringReader sr = new StringReader(clean(metadata.m_metadata));
           int len = metadata.m_metadata.length(); 
         mainstmt.setCharacterStream (5, sr, len );
	  
         // mainstmt.setString(5,clean(metadata.m_metadata)); 
       }catch (SQLException sqle){
           System.out.println("Set String to Long !!!!" + sqle.getMessage());
       } 
       int i = mainstmt.executeUpdate();
       return i;
    
    }

    private int freeUpdate(dienst.Record record) throws SQLException{

        freestmt.setString(1,record.m_id);
        freestmt.setString(2,record.m_archive);
        freestmt.setString(3,record.toString().trim());
          int date=0;
       try{
            date=Integer.parseInt(record.m_datestamp.substring (0,4));
       }catch (Exception e){
	   System.out.println(e);
       }

        freestmt.setInt(4,date);
        freestmt.setString(5,record.m_datestamp);
        int i=freestmt.executeUpdate();
        return i;
    }

    private String clean(String s){
        if (s==null)
            return "";

       	StringBuffer buffer=new StringBuffer();
   	    for (int i=0;i<s.length();i++){
                if (s.charAt(i)=='\'')
                    buffer.append(" ");
                else
                    buffer.append(s.charAt(i));
	        }
    	return buffer.toString().trim();
    }

    public abstract void reindex();

    private int DCUpdate(dienst.Record record,boolean cached) throws SQLException{

       dcstmt.setString(1,clean(record.m_id));
       dcstmt.setString(2,clean(record.m_archive));
       dcstmt.setString(3,clean(record.m_datestamp));
       dcstmt.setString(4,clean(record.m_sets));
       dcstmt.setString(5,clean(record.m_title));
       dcstmt.setString(6,clean(record.m_creator));
       dcstmt.setString(7,clean(record.m_subject));
       dcstmt.setString(8,clean(record.getDescription()));
       dcstmt.setString(9,clean(record.m_publisher));
       dcstmt.setString(10,clean(record.m_contributor));
       dcstmt.setString(11,clean(record.m_discovery));
       dcstmt.setString(12,clean(record.m_type));
       dcstmt.setString(13,clean(record.m_format));
       dcstmt.setString(14,clean(record.m_identifier));
       dcstmt.setString(15,clean(record.m_source));
       dcstmt.setString(16,clean(record.m_language));
       dcstmt.setString(17,clean(record.m_relation));
       dcstmt.setString(18,clean(record.m_coverage));
       dcstmt.setString(19,clean(record.m_rights));
       if (cached)
            dcstmt.setString(21,"Y");
       else
            dcstmt.setString(21,"N");
       int date=0;
       try{
            date=Integer.parseInt(record.m_datestamp.substring (0,4));
       }catch (Exception e){
	   System.out.println(e);
       }

       dcstmt.setInt(20,date);
       int i=dcstmt.executeUpdate();
       return i;

   }



    public void add(String archivename,dienst.RecordList rList){
	   System.out.println("Size "+ rList.size());
	   if (rList.size()==0)
	       return;
	   initstmt();

            System.out.println("Size "+ rList.size());
            for (int i=0;i<rList.size();i++)
                add(archivename,(Record)rList.get(i));


	    
            closeConnection();

    }



    public void add(String archivename,dienst.Record r){
	if (r==null)
	    return;
        boolean useflag=false;
        if (connection==null){
            initstmt();
            useflag=true;
        }

        boolean cached=cache(r);
        delete(r);

	 try{
            DCUpdate(r,cached);
            freeUpdate(r);
        }
        catch(SQLException e)
        {
            System.out.println(e.getErrorCode()+e.toString());
            String errmsg=r.m_id+ "--"+ e.getErrorCode()+e.toString();
	    log.FileLog.writeError(errmsg);
	    
         
        }
	    if (r.m_sets!=null)
               updatePartition(r);
        if (useflag)
            closeConnection();

    }

   /** Added by Yang Zhao Mar 2002 */
    public void add(dienst.MetadataList mList){
	   System.out.println("Size "+ mList.size());
	   if (mList.size()==0)
	       return;
	   initstmt();

           System.out.println("Size "+ mList.size());
           for (int i=0;i<mList.size();i++)
                add((Metadata)mList.get(i));
	    
            closeConnection();

    }    
    /** Added by Yang Zhao Mar 2002 */
     public void add(dienst.Metadata m){
	if (m==null)
	    return;
        boolean useflag=false;
        if (connection==null){
            initstmt();
            useflag=true;
        }

	try{
            delete(m);
            mainUpdate(m);
   
        }
        catch(SQLException e)
        {
            System.out.println(e.getErrorCode()+e.toString());
            String errmsg=m.m_identifier+ "--"+ e.getErrorCode()+e.toString();
	    log.FileLog.writeError(errmsg);
	    
         
        }
        if (useflag)
            closeConnection();

    }

    private boolean cache(Record r){

        Statement s=null;
        boolean result=false;
        try{
            String flag=null;
            s=connection.createStatement();
            ResultSet rset= s.executeQuery("select cache from dc where id='"+clean(r.m_id.trim())+"'");
            while (rset.next()){
                flag=rset.getString("cache");
            }

            if ((flag!=null)&&(flag.equals("Y")))
                return true;

            if (Config.CACHE_DIR ==null)
                return false;

                String filename=r.m_id.replace('/','.');
                filename=filename.replace('\\','.');
                filename=filename.replace(':','.');
                filename=filename.replace(' ','.');
                File file=new File(Config.CACHE_DIR+File.separatorChar+filename+r.m_identifier.substring(r.m_identifier.lastIndexOf(".")));

                FileOutputStream bufferOut=new FileOutputStream(file);
                URL url=new URL(r.m_identifier);

                byte[] buffer=new byte[10000];
                InputStream in =url.openStream();
                BufferedInputStream bufferIn=new BufferedInputStream(in);
                int len=0;
                while((len= bufferIn.read(buffer,0,10000))!=-1)  {
                   bufferOut.write(buffer,0,len);
                }

                bufferOut.close();
                result=true;
       }

        catch (Exception e){
            System.out.println(e);
            result=false;
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }
        return result;

    }

    private void delete(Record r){
        Statement s=null;
        try{
            s=connection.createStatement();

            try{
              s.executeUpdate("delete from dc where id='"+clean(r.m_id.trim())+"'");
            }
            catch(Exception ex){
            }

            try{
              s.executeUpdate("delete from free where id='"+clean(r.m_id.trim())+"'");
            }
            catch(Exception ex){
            }

        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }


    }

     private void delete(dienst.Metadata md){
        Statement s=null;
        try{
            s=connection.createStatement();

            try{
              s.executeUpdate("delete from main where id='"+clean(md.m_identifier.trim())+"'");
            }
            catch(Exception ex){
            }

        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }


    }

    public void updatePartition(Record r){
        Statement s=null;
        try{
            s=connection.createStatement();
            ResultSet rset= s.executeQuery("select sets from dc where id='"+r.m_id.trim()+"'");
            rset.next();
            String oldpartition=rset.getString(1);

            if (oldpartition==null){

                String command="update dc set sets='"+r.m_sets+"' where id='"+r.m_id.trim()+"'";
                s.executeUpdate("update dc set sets='"+r.m_sets+"' where id='"+r.m_id.trim()+"'");
            }
            else  if (oldpartition.indexOf(r.m_sets)==-1){
                s.executeUpdate("update dc set sets='"+oldpartition+" | "+r.m_sets+"' where id='"+r.m_id.trim()+"'");
            }
        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }

    }


     public void updateIdentify(IdentifyInfo info){
        boolean useflag=false;
        Statement querys=null,updates=null;
        PreparedStatement ps=null;

        try{
           connection=m_connectionPool.getConnection();
           querys=connection.createStatement();
           ResultSet rs=querys.executeQuery("select id from dpinfo where id='"+clean(info.id)+"'");
           if (!rs.next()){
               String query="insert into dpinfo(id,BASEURL,"+
               "homepage,NAME,VERSION,ADMINEMAIL,DELIMITER,SAMPLEIDENTIFIER,SCHEMA,datestamp) values (?,?,?,?,?,?,?,?,?,?)";
               ps = connection.prepareStatement(query);
               ps.setString(1,clean(info.id));
               ps.setString(2,clean(info.baseURL ));
               ps.setString(3,clean(info.homepage));
               ps.setString(4,clean(info.name));
               ps.setString(5,clean(info.version ));
               ps.setString(6,clean(info.adminemail));
               ps.setString(7,clean(info.delimiter ));
               ps.setString(8,clean(info.sampleIdentifier ));
               ps.setString(9,clean(info.schema));
            
               SimpleDateFormat formatter
                   = new SimpleDateFormat ("yyyy-MM-dd");
                 java.util.Date currentTime_1 = new java.util.Date();
              String dateString = formatter.format(currentTime_1);

               ps.setString(10,dateString);
               ps.executeUpdate();
           }
           //existing data provider
           else{

	       String query="update dpinfo set version=?,name=?,baseURL=? where id=?";
               ps = connection.prepareStatement(query);
               ps.setString(1,clean(info.version));
	       ps.setString(2,clean(info.name));
               ps.setString(3,clean(info.baseURL ));
	       ps.setString(4,clean(info.id));
               ps.executeUpdate();
	   

           }

        }
        catch (SQLException e){
            System.out.println(e.getErrorCode()+e.toString ());

        }
        finally{
            try{
                if(querys!=null)
                    querys.close ();
                if (updates!=null)
                    updates.close();
                if (ps!=null)
                    ps.close();
                m_connectionPool.close(connection);
                connection=null;
            }
            catch(Exception e){
                System.out.println(e);
            }

        }


    }


    public void updatePartition(RecordList list){
          for (int i=0;i<list.size();i++)
              updatePartition((Record)list.get(i));
    }

   public void updateHarvestmethod(String identifier){        
	try{
            Connection conn = m_connectionPool.getConnection();            
            Statement s=conn.createStatement();
             
            s.executeUpdate("update dplist set harvestmethod='fresh_mass' where identifier='" + identifier+"'");
            s.close();
            m_connectionPool.close(conn);
        }
        catch (SQLException e){
            System.out.println("Update Harvest Method: " + e);
        }
    }
}








