package scheduler;
 
import java.util.*; 
import java.text.*; 
import dienst.*; 
 
import db.*; 
import ns.*; 
import ns.ldap.*; 
import interactive.createSubject.*;
 
import log.*;
 
public class Scheduler{ 
    Vector taskList; 
    Record2DB o=null; 
    ConnectionPool m_connectionPool=null; 
    Log m_log = null;
    boolean timer=false;

    Config c; 
    public Scheduler(Config c, ConnectionPool cpool) { 
        this.c=c; 
        taskList=new Vector(); 
    
        m_connectionPool = cpool;
        /**
        m_connectionPool=new ConnectionPool(); 
        try{ 
            m_connectionPool.initialize();
        }
        catch(Exception ex){
            ex.printStackTrace();
        }
	*/
	if (m_connectionPool.getDBType()==ConnectionPool.MYSQL)
            o=new Record2Mysql(m_connectionPool);
        else  if (m_connectionPool.getDBType()==ConnectionPool.ORACLE)
            o=new Record2Oracle(m_connectionPool); 
 
    } 
 
 
    public void addTask(Task t){ 
        taskList.add(t); 
    } 
 
    public void clear(){ 
        taskList.clear(); 
    } 
 
 
    public void prepare(){ 
 
        //check supported database
        clear();
       
            //load data provider list 
        NameList dpreader=null;
        System.out.println("data source="+c.DP_LIST_SOURCE);
        if (c.DP_LIST_SOURCE.equals("xml")){
            dpreader=new XMLDPReader();
        }
        else if (c.DP_LIST_SOURCE.equals("database")){
            dpreader=new DBDPReader(m_connectionPool);
        }
        else if (c.DP_LIST_SOURCE.equals("ldap")){
            dpreader=new LDAPDPReader();
        }

        dpreader.init();
//	 Hashtable providerList=(Hashtable)dpreader.getNameList();
	 Vector providerList=(Vector)dpreader.getNameList();

	m_log = new Log(m_connectionPool);
//	 for(Iterator i=providerList.values().iterator();i.hasNext();){
	 for(Iterator i=providerList.iterator();i.hasNext();){
            DPName name=(DPName)i.next();
            if (name.active && (name.oaiversion.trim()).startsWith("1")){
                DataProvider dp=new DataProvider(name.id,name.baseURL,name.sets,name.lastharvesttime, name.metaFormat);
                Task h=null;
	       
                if (name.harvestMethod.equals ("fresh_mass"))
                    h=new HarvestListAddNew(dp,o,m_log);
                else if (name.harvestMethod.equals("fresh_single"))
                    h=new HarvestDmtAddNew(dp,o, m_log);
                else if(name.harvestMethod.equals("history_mass"))
                    h=new HarvestListAddAll(dp,o, m_log);
                else if (name.harvestMethod.equals("history_single"))
                    h=new HarvestDmtAddAll(dp,o, m_log);
                
		if (h!=null) 
		    this.addTask (h);

            }
        }

	
    } 


    public void timerOn(){
	timer=true;
    }


    public void reindex(){
	System.out.println("--- Create interactive search table");
	CreateDB interactiveDB=new CreateDB();
	interactiveDB.run(m_connectionPool);
	System.out.println("--- Reindex Database");
	o.reindex();
    }

    public void harvest(){ 
          for (;;){ 
            prepare(); 
            for (Enumeration e = taskList.elements() ; e.hasMoreElements() ;) { 
                  Task t=(Task)(e.nextElement()); 
                  try{ 
                      t.prepare(); 
                  } 
                  catch(Exception ex){ 
                    System.out.println("in prepare error"+ex);
		    LogRecord m_logrecord = t.getLog();
		    if (m_logrecord == null) 
			m_logrecord = new LogRecord(Calendar.getInstance().getTime(),"Prepare task", t.toString(), "Require",false);

		    m_logrecord.m_status = "Error";
		    m_logrecord.m_detail = ex.toString();
		    m_log.writeErrorLog(m_logrecord);
                    continue; 
                  } 
 
                  for (;!t.isDone();){ 
                    try{ 
                          t.run(); 
                      } 
                      catch (Exception ex){ 
			  LogRecord m_logrecord = t.getLog();
			  if (m_logrecord != null) 
			      m_logrecord = new LogRecord(Calendar.getInstance().getTime(),"Run task", t.toString(), "Db",false);
			  m_logrecord.m_status = "Error";
			  m_logrecord.m_detail = ex.toString();
			  m_log.writeErrorLog(m_logrecord);
                          System.out.println(ex); 
                        } 
 
                 
 
                  } 
                  System.out.println("Finish task: "+t); 
 
             }//end all task 
 
 
             for (Enumeration e = taskList.elements() ; e.hasMoreElements() ;) { 
                   Task t=(Task)(e.nextElement()); 
                   if (!t.forever()) 
                       taskList.remove(t); 
              } 
	     
	     System.out.println("Finish Harvesting task:"+ Clock.today());
	    
	     if (timer==true){
		 System.out.println("Finish "+Clock.today ()+" begin to sleep"); 
		 try{ 
		 //Sleep 10 hours, for the possible delay 
		     Thread.sleep (1000*60*Config.SLEEP_BETWEEN_HARVEST); 
		 } 
		 catch (Exception e){ 
		     System.out.println(e); 
		 } 
	     
		 taskList.clear ();
	     }
	     else
		 System.exit(0);
	  }//end all 
	  
    }//end run 
    
} 


