package log;


import java.lang.*;
import java.text.*;
import java.sql.*;

import db.*;


/**
 * Log class 
 */
public class Log {
    
    ConnectionPool m_pool = null;
    private int dbtype;
    /**
     * Construct a new Log
     * @param pool Connection pool
     */
    public Log(ConnectionPool pool) {
	m_pool = pool;
        dbtype=m_pool.getDBType();
    }
    /**
     * write the beginning of the log
     * @param record the Log record
     */
    public int writeBeginLog(LogRecord record)
    {
	int i =0;
	try{
	    Connection connection=m_pool.getConnection();
	    PreparedStatement logstmt = connection.prepareStatement("insert into harvestlog(inputdate,operation,source,destination,status) values (?,?,?,?,?)");
	    SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss:SSS");
	    String datetime = formatter.format(record.m_inputdate);
	    logstmt.setString(1,datetime);
	    logstmt.setString(2,record.m_operation);
	    logstmt.setString(3,record.m_source);
	    logstmt.setString(4,record.m_destination);
	    logstmt.setString(5,record.m_status);
	    i=logstmt.executeUpdate();
	    m_pool.close(connection);
        }catch(Exception e){
	    System.out.println(e);
        }
	return i;

    }

    /**
     * write the end of the Log
     * @param record the log record
     * @param identify 
     * @return number of updated records
     */
    public int writeEndLog(LogRecord record, String identify)
    {
	int i=0;
	try{
	    Connection connection=m_pool.getConnection();
	    PreparedStatement logstmt = connection.prepareStatement("update harvestlog set status = ?,totalnum=?, detail=?  where inputdate = ?");
	    SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss:SSS");
	    String datetime = formatter.format(record.m_inputdate);
	    logstmt.setString(1,record.m_status);
	    logstmt.setInt(2,record.m_totalnum);
	    logstmt.setString(3, record.m_detail);
	    logstmt.setString(4,datetime);
	    i=logstmt.executeUpdate();

	    if (record.m_status.equals("OK") && record.m_updateharvesttime) {

		/**
                String s = toString(record.m_inputdate,"yyyy-MM-dd HH:mm:ss");
                String format ="YYYY-MM-DD HH24:MI:SS"; //for SQL date format
            
                String updateStr=null;
                if (dbtype == ConnectionPool.ORACLE)  
	          updateStr = "update dplist set lastharvesttime = to_date('"+ s + "', '" + format +"') where identifier = '" + identify +"'";
                else
                  updateStr = "update dplist set lastharvesttime='" + s + "' where identifier = '" + identify +"'";
		PreparedStatement dpliststmt = connection.prepareStatement(updateStr);
		*/
		
	       
               PreparedStatement dpliststmt = connection.prepareStatement("update dplist set lastharvesttime = ? where identifier = ?");

	       dpliststmt.setTimestamp(1, new java.sql.Timestamp(record.m_inputdate.getTime()));
	       dpliststmt.setString(2,identify);       

		i = dpliststmt.executeUpdate();

	    }
	    m_pool.close(connection);
        }catch(Exception e){
	     System.out.println(e);
        }
	return i;

    }
    /**
     * write error log
     * @param record the log record
     * @return number of updated records
     */
    public int writeErrorLog(LogRecord record) {
	int i=0;
	try{
	    SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss:SSS");
	    String datetime = formatter.format(record.m_inputdate);
	    Connection connection=m_pool.getConnection();
	    Statement stmt = connection.createStatement();
	    ResultSet rs = stmt.executeQuery("select count(*) as tt from harvestlog where inputdate = '"+datetime+"'");
	    PreparedStatement logstmt = null;
	    if (rs.next()) {
		logstmt = connection.prepareStatement("update harvestlog set status = ?,detail=?  where inputdate = ?");
		logstmt.setString(1,record.m_status);
		logstmt.setString(2, record.m_detail);
		logstmt.setString(3,datetime);
	    } else {
		logstmt = connection.prepareStatement("insert into harvestlog (inputdate,operation,source,destination,status, detail) values (?,?,?,?,?,?)");
		logstmt.setString(1,datetime);
		logstmt.setString(2,record.m_operation);
		logstmt.setString(3,record.m_source);
		logstmt.setString(4,record.m_destination);
		logstmt.setString(5,record.m_status);
		logstmt.setString(6,record.m_detail);
	    }
	    i=logstmt.executeUpdate();
	    rs.close();
	    stmt.close();
	    m_pool.close(connection);
        }catch(Exception e){
            System.out.println(e);
        }
	return i;
     }

    private String toString (java.util.Date date, String pattern){
        SimpleDateFormat formatter  = new SimpleDateFormat (pattern);
        return formatter.format(date);
    }
}


