package ns;

import java.io.*;
import java.sql.*;
import java.util.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.text.*;


import db.*;
/**
 * This class fetches data provider list from database.
 *
 * database schema: create table dplist(identifier varchar(20) primary key, baseurl varchar(100), active char(1),
 * harvestmethod varchar(20));
 * Used by Harvesting Tools.
 *
 * Author xiaoming Liu (liu_x@cs.odu.edu)
 */

public class DetailDPReader implements NameList
{
     private Hashtable hash;
      private Vector list;
    ConnectionPool m_connectionPool=null;
    
    SimpleDateFormat formatter
     = new SimpleDateFormat ("yyyy.MM.dd G 'at' hh:mm:ss a zzz");
 
    /**
     * Default Constructor --
     * Construct a new data provider reader that inherits Hashtable class
     *
     */
    public DetailDPReader(ConnectionPool cpool)
    {
         hash=new Hashtable();
       list = new  Vector();
         m_connectionPool=cpool;
    }


    /**
     * Parsing the specified XML document containing the information of data providers
     *
     * @param xmlUri the uri of XML document
     * @return None
     */
    public void init() {
        Statement stmt=null;

        Connection connection=null;

        try{

            connection=m_connectionPool.getConnection();
            stmt=connection.createStatement();
            ResultSet rs=stmt.executeQuery("select dpinfo.*,harvestmethod,lastharvesttime from dpinfo,dplist where dpinfo.id=dplist.identifier");
            while(rs.next()){
                DPInfo dp=new DPInfo();
                dp.id = rs.getString("id");
                dp.homepage=rs.getString("homepage");
                dp.baseURL = rs.getString("baseURL");
                dp.name=rs.getString("name");
		if (dp.name==null)
		    dp.name=dp.id;
                dp.version=rs.getString("version");
                dp.adminemail=rs.getString("adminemail");
                dp.delimiter=rs.getString("delimiter");
                dp.sampleIdentifier=rs.getString("sampleIdentifier");
                dp.schema=rs.getString("schema");
		dp.harvestmethod=rs.getString("harvestmethod");
		dp.lastharvesttime=(rs.getTimestamp("lastharvesttime")==null)?null:formatter.format(rs.getTimestamp("lastharvesttime"));
		
                hash.put(dp.id,dp);
                list.add(dp);
            }


        }catch(Exception e){
            System.out.println(e);
        }
       finally{
            try{
                stmt.close();
                m_connectionPool.close(connection);
            }
            catch(Exception e){
                System.out.println(e);
            }
        }

    }

    /**
     * Return parse results
     */

     public Vector getNameList(){
        return list;
     }

	public Hashtable getHashedNamelist(){
	return hash;
	}
    /**
     * This main function demonstrates class DPReader
     * @param args the URI of the XML document
     * @return None
     */
    public static void main(String args[]) throws java.lang.Exception
    {

        ConnectionPool cpool=new ConnectionPool();
        cpool.initialize ();
    	DetailDPReader dpr = new DetailDPReader(cpool);
        dpr.init();
	    for (Enumeration keys = dpr.getNameList().elements(); keys.hasMoreElements();)
    	    System.out.println(keys.nextElement());

    }
}






