package oairequester;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.*;

class ListRecordsHandler extends DefaultHandler{
    MetadataList metadataList;
    Vector deletedList; 

    Metadata md =null;    
    StringBuffer record=null;
    StringBuffer identifier=null;
    StringBuffer resumptionToken=null;
  
    String archive="";
    String format="";
    String partition="";
 
    boolean deletedFlag=false;
    boolean headerFlag=false;
    boolean recordFlag = false;
    boolean tokenFlag = false;
    boolean idFlag = false;

    public static final String OAI_NS="http://www.openarchives.org/OAI/2.0/";
    public static final String OAI_METADATA=fullName(OAI_NS, "metadata");
    public static final String OAI_IDENTIFIER = fullName(OAI_NS, "identifier");
    public static final String OAI_SETSPEC = fullName(OAI_NS, "setSpec");
    public static final String OAI_DATESTAMP = fullName(OAI_NS, "datestamp");
    public static final String OAI_RECORD = fullName(OAI_NS, "record");
    public static final String OAI_HEADER = fullName(OAI_NS, "header");
    public static final String OAI_RESUMPTION_TOKEN = fullName(OAI_NS, "resumptionToken");


    ListRecordsHandler (String format,String archive, String partition, InputSource input) throws java.io.IOException{
	super();
	this.format=format;
	this.archive=archive;
	this.partition=partition;
	metadataList = new MetadataList();
	deletedList=new Vector();

	try {
	    SAXParserFactory factory = SAXParserFactory.newInstance();
	    factory.setNamespaceAware(true);
            factory.setFeature("http://xml.org/sax/features/namespaces", true);
	    factory.setFeature("http://xml.org/sax/features/namespace-prefixes", true);	    
	    SAXParser saxParser= factory.newSAXParser();
	    saxParser.parse(input,this);
	} catch (ParserConfigurationException e) {
	    e.printStackTrace();
	} catch (SAXException e) {
	    e.printStackTrace();
	}   
    }
    

    public void startElement(String uri, String localName, String rawName, Attributes attrs)
	throws SAXException{
	String fullName=fullName(uri,localName);
	if (fullName.equals(OAI_RECORD)){
            record=new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
            recordFlag = true;	    
            md = new Metadata();         	  
	}
        else if (fullName.equals(OAI_RESUMPTION_TOKEN)){
            tokenFlag = true;
	    resumptionToken=new StringBuffer("");
	}
	else if (fullName.equals(OAI_HEADER)){
            headerFlag = true;
	    int length = attrs.getLength();
	    for (int i=0; i<length; ++i) {	
		String aEName = attrs.getQName(i);		
		if (aEName.equals("status") && (attrs.getValue(i).equals("deleted")))
		    deletedFlag=true;
	    }
	}
	else if(fullName.equals(OAI_IDENTIFIER)){
	    idFlag = true;	  
	    identifier=new StringBuffer("");
	}
	

        if (recordFlag){
	    record.append("<");
	    record.append(rawName);
	    int length = attrs.getLength();
	    for (int i=0; i<length; ++i) {
		record.append(" ");
		String aEName = attrs.getQName(i);
		record.append(aEName);
		record.append("=\"");
		record.append(xmlEncode(attrs.getValue(i)));
		record.append("\"");
	    }
	    record.append(">");	  
	}
	
    }

    public void endElement (String uri, String localName,String rawName) throws SAXException {
	String fullName = fullName(uri, localName);

	if (recordFlag){
	    record.append("</");
	    record.append(rawName);
	    record.append(">");         
	}
	if (fullName.equals(OAI_RECORD)){
	    if (deletedFlag)
		deletedList.add(identifier.toString());
	    else{
		md.setMetadata(record.toString());
		recordFlag = false;           
		md.setIdentifier(identifier.toString());
		md.setFormat(format);
		md.setArchive(archive);
		md.setSets(partition);
                md.setTimestamp( MyClock.currentTime() );
		metadataList.add(md);		
	    }
	    identifier = null;	
	    md =null;
	}
        if(fullName.equals(OAI_IDENTIFIER))
	    idFlag = false;      
	if (fullName.equals(OAI_RESUMPTION_TOKEN))
	    tokenFlag = false;
	else if (fullName.equals(OAI_HEADER)){
	    headerFlag=false;
	}
    }

    public void characters (char[] ch, int start, int end) throws SAXException {
	String s = new String (ch, start, end);      
        if (recordFlag){
	    record.append(xmlEncode(s));
	}
        if (idFlag)
	    identifier.append(s);      
        if (tokenFlag)
	    resumptionToken.append(s);

    }
   
    public MetadataList getMetadataList(){
	return metadataList;
    }

    public Vector getDeletedList(){
	return deletedList;
    }

    public String getResumptionToken(){
	if (resumptionToken==null)
	    return null;
	else
	    return resumptionToken.toString();
    }


    public static String fullName(String namespaceURI, String localName) {
	StringBuffer sb = new StringBuffer();
	sb.append(namespaceURI);
	sb.append("#");
	sb.append(localName);
	return sb.toString();
    }
    
    public static String xmlEncode(String s) {
        StringBuffer sb = new StringBuffer();

        for (int i=0; i<s.length(); ++i) {
            char c = s.charAt(i);
            switch (c) {
            case '&':
                sb.append("&amp;");
                break;
            case '<':
                sb.append("&lt;");
                break;
            case '>':
                sb.append("&gt;");
                break;
            default:
                sb.append(c);
                break;
            }
        }
        return sb.toString();
    }
 }
