package parser;
import org.w3c.dom.*;
import org.xml.sax.*;
import oairequester.*;
/**
 * Parse Dublin Core record
 */
public class DCMetaParser extends MetaParser{
    /** 
     * constructor
     */
     public DCMetaParser(){
        super();
     } 
    /**
     * parse 
     * @return Result record
     */  
     public Result parseOAMS() {
        if (recordElement==null)
            return null;
        Record record=new Record();
         
        record.m_archive = archive;
        record.m_sets = sets;
 
        NodeList recordChild=recordElement.getChildNodes();

        boolean headerflag=false;
        Element header=null,dc=null,about=null,oamsElement=null;
        for (int m=0;m<recordChild.getLength();m++){
            Node childOfRecord=recordChild.item(m);
            if(childOfRecord.getNodeType()==Node.ELEMENT_NODE)
               if (childOfRecord.getNodeName ().equalsIgnoreCase ("header")){
                    header=(Element)(childOfRecord);
                    headerflag=true;
                  
                }
               else if (childOfRecord.getNodeName ().equalsIgnoreCase ("metadata")){
                   dc=(Element)(childOfRecord);
               }
               else if (childOfRecord.getNodeName ().equalsIgnoreCase ("about")){
                    about=(Element)(childOfRecord);
                }
        }


        if ((header==null)||(dc==null))
            return null;

        NodeList nl=header.getChildNodes();
        for (int k=0;k<nl.getLength();k++){
            Node test=nl.item(k);
            if (test.getNodeType()==Node.ELEMENT_NODE)
            {
                Element itemElement=(Element)test;
                if (itemElement.getTagName().endsWith("identifier"))
                    record.m_id=fetchChild(itemElement);
                if (itemElement.getTagName().endsWith("datestamp"))
                    record.setDateStamp(fetchChild(itemElement));
            }
        }


        //processing dc field

        nl=dc.getChildNodes ();
         for (int k=0;k<nl.getLength();k++){
            Node test=nl.item(k);
            if (test.getNodeType()==Node.ELEMENT_NODE)
            {
                oamsElement=(Element)test;
                break;
            }
        }

        if (oamsElement==null)
            return null;

        NodeList oams=oamsElement.getChildNodes();

        for (int k=0;k<oams.getLength();k++){
            Node test=oams.item(k);
            if (oams.item(k).getNodeType()==Node.ELEMENT_NODE)
            {
                Element itemElement=(Element)oams.item(k);
                if (itemElement.getTagName().endsWith("title"))
                    record.setTitle(calculateValue(record.m_title,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("creator"))
                    record.setCreator(calculateValue(record.m_creator,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("subject"))
                    record.setSubject(calculateValue(record.m_subject,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("description"))
                    record.setDescription(calculateValue(record.getDescription(),fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("publisher"))
                    record.setPublisher(calculateValue(record.m_publisher,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("contributor"))
                    record.setContributor(calculateValue(record.m_contributor,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("date"))
                    record.setDiscovery(calculateValue(record.m_discovery,fetchChild(itemElement)));
		               else if (itemElement.getTagName().endsWith("type"))
				  record.setType(calculateValue(record.m_type,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("format"))
                    record.setFormat(calculateValue(record.m_format,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("identifier"))
                    record.setIdentifier(calculateValue(record.m_identifier,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("source"))
                    record.setSource(calculateValue(record.m_source,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("language"))
                    record.setLanguage(calculateValue(record.m_language,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("relation"))
                    record.setRelation(calculateValue(record.m_relation,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("coverage"))
                    record.setCoverage(calculateValue(record.m_coverage,fetchChild(itemElement)));
                else if (itemElement.getTagName().endsWith("rights"))
                    record.setRights(calculateValue(record.m_rights,fetchChild(itemElement)));

            }
        }//end for
        
        
        return record;
    }


    private static String calculateValue(String old, String add){
        if ((old==null)||old.equals(""))
            old=add;
        else
            old=old+" | "+add;
        return old;
    }

    /**
     * fetch child
     * @param e Element
     * @return string
     */
    public static String fetchChild(Element e){
        if (e==null)
            return null;

        NodeList nl=e.getChildNodes();
        if (nl.getLength()!=0){
            String result="";
            for (int i=0;i<nl.getLength();i++){
                Node n=nl.item(i);
//
//Special processing for ENTITY_REFERNCE_NODE
//
                if ((n.getNodeType()==Node.ENTITY_REFERENCE_NODE) ||(n.getNodeType ()==Node.ENTITY_NODE))
                {
                    NodeList entitynl =n.getChildNodes ();
                    for (int s=0;s<entitynl.getLength();s++){
                        result+=entitynl.item(s).getNodeValue();
                    }
                }
                else
                    result+=nl.item(i).getNodeValue();
                    /*if (result.length ()>=14){
                    System.out.println(Character.digit (result.charAt (14),16));
                    System.out.println(Character.isDefined (result.charAt (14)));}*/
            }
            return result.trim();
        }
        else
            return null;
    }


}










