package parser;


import java.sql.*;
import java.io.*;

import java.net.*;
import java.util.*;
import oairequester.*;
import java.text.*;
import ns.*;
import db.*;

/** insert record into DB, abstract class, has to be inherited */
public abstract  class Record2DB{
    ConnectionPool m_connectionPool=null;
    Connection connection=null;
    PreparedStatement freestmt=null,tablestmt=null, mainstmt=null;
    String tableName = null;
    int idexist=0;
    ns.Cache cachecfg=null;
    
    DateNormalizer dn=new DateNormalizer();
    /** 
     *constructor
     *@param pool ConnectionPool
     *@param destin String
     */

    public Record2DB(ConnectionPool pool, String destin){
        m_connectionPool=pool;
        tableName = destin;
	cachecfg=(new CacheCfg(pool)).getCache();
        
    }
    /**
     * init sql statement
     */
    public void initstmt(){
        String insertClause = "insert into " + tableName;
        if (tableName.endsWith("dc"))
	    insertClause += " (id,archive,datestamp,sets,title,creator,subject,description,publisher,contributor,discovery,type,format,identifier,source,language,relation,coverage,rights,groupdate,cache,status, normdiscovery) values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)" ;
	
	
        try{
	    connection=m_connectionPool.getConnection();
	    freestmt = connection.prepareStatement("insert into free(id,archive,alltext,groupdate,datestamp,normdiscovery,subject) values (?,?,?,?,?,?,?)");
	    tablestmt = connection.prepareStatement(insertClause);
	    
        }catch(Exception e){
            System.out.println(e);
        }
	
   }
    /**
     * close conn
     */
    public void closeConnection(){

	try{
	    freestmt.close();
	    tablestmt.close();
	    //mainstmt.close(); 
	}
	catch(Exception e){
	   System.out.println(e);
	}
	
	if (connection!=null)
            m_connectionPool.close(connection);
	
        connection=null;
    }
    
    
    
    private int freeUpdate(oairequester.Record record) throws SQLException{
	
        freestmt.setString(1,record.m_id);
        freestmt.setString(2,record.m_archive);
        freestmt.setString(3,record.toString().trim());
	int date=0;
	try{
            date=Integer.parseInt(record.m_datestamp.substring (0,4));
	    freestmt.setString(6,clean(dn.format(record.m_discovery)));
       }catch (Exception e){
	   System.out.println(e);
       }
	
        freestmt.setInt(4,date);
        freestmt.setString(5,record.m_datestamp);
	freestmt.setString(7,clean(record.m_subject));
	
        int i=freestmt.executeUpdate();
        return i;
    }
    
    private String clean(String s){
        if (s==null)
            return "";
	
       	StringBuffer buffer=new StringBuffer();
	for (int i=0;i<s.length();i++){
	    if (s.charAt(i)=='\'')
		buffer.append(" ");
	    else
		buffer.append(s.charAt(i));
	}
    	return buffer.toString().trim();
    }
    /** reindex */
    public abstract void reindex();
    
    private int DCUpdate(oairequester.Record record,boolean cached) throws SQLException{
	
	tablestmt.setString(1,clean(record.m_id));
	tablestmt.setString(2,clean(record.m_archive));
	tablestmt.setString(3,clean(record.m_datestamp));
	tablestmt.setString(4,clean(record.m_sets));
	tablestmt.setString(5,clean(record.m_title));
	tablestmt.setString(6,clean(record.m_creator));
	tablestmt.setString(7,clean(record.m_subject));
	tablestmt.setString(8,clean(record.getDescription()));
	tablestmt.setString(9,clean(record.m_publisher));
	tablestmt.setString(10,clean(record.m_contributor));
	tablestmt.setString(11,clean(record.m_discovery));
	tablestmt.setString(12,clean(record.m_type));
	tablestmt.setString(13,clean(record.m_format));
	tablestmt.setString(14,clean(record.m_identifier));
	tablestmt.setString(15,clean(record.m_source));
	tablestmt.setString(16,clean(record.m_language));
	tablestmt.setString(17,clean(record.m_relation));
	tablestmt.setString(18,clean(record.m_coverage));
	tablestmt.setString(19,clean(record.m_rights));
	if (cached)
            tablestmt.setString(21,"Y");
	else
            tablestmt.setString(21,"N");
       int date=0;
       try{
	   date=Integer.parseInt(record.m_datestamp.substring (0,4));
       }catch (Exception e){
	   System.out.println(e);
       }
       
       tablestmt.setInt(20,date);
       tablestmt.setString(22,clean(record.m_status));
       tablestmt.setString(23,clean(dn.format(record.m_discovery)));
       int i=tablestmt.executeUpdate();
       
       return i;
       
    }

    
    /** 
     * add list
     * @param rlist RecordList
     */
    
    public void add(oairequester.RecordList rList){
	   System.out.println("List Size "+ rList.size());
	   if (rList.size()==0)
	       return;
	   initstmt();

	   System.out.println("Size "+ rList.size());
	   for (int i=0;i<rList.size();i++)
	       add((Record)rList.get(i));
	 	    
	   closeConnection();

    }

    /**
     * add record
     *@param r Record 
     */
    public void add(oairequester.Record r){
	if (r==null)
	    return;
        boolean useflag=false;
        initstmt();
        boolean cached=cache(r);
	
        Statement s =null;
        try{
	    s=connection.createStatement();
	    ResultSet rset = null;
	    rset= s.executeQuery("select id, datestamp from dc where id='"+clean(r.m_id.trim())+"'");
	    if (rset.next()){
		String olddatestamp = rset.getString(2);
		
		if (r.m_datestamp.compareTo(olddatestamp) > 0 )
		    writeUpdate(r, olddatestamp);
	    }		  
	    
	    delete(r);
	   
	    DCUpdate(r,cached);
	    
	    freeUpdate(r);
	    
        }
        catch(SQLException e)
        {
	    System.out.println(e.getErrorCode()+e.toString());
	    String errmsg=r.m_id+ "--"+ e.getErrorCode()+e.toString();
	    log.FileLog.writeError(errmsg);
	}
        catch(Exception ex){}
        
        if (r.m_sets!=null)
            updatePartition(r);
            closeConnection();
	
    }
    
    
   
    
    private boolean cache(Record r){
	
        Statement s=null;
        boolean result=false;
        try{
            String flag=null;
            s=connection.createStatement();
            ResultSet rset= s.executeQuery("select cache from dc where id='"+clean(r.m_id.trim())+"'");
            if (rset.next()){
                flag=rset.getString("cache");
            }
	    
            if ((flag!=null)&&(flag.equals("Y")))
                return true;
	    
            if (cachecfg.iscache.equals("N"))
                return false;
	    URL url = null;
	    String filename=null;
	    
	    StringTokenizer st=new StringTokenizer(r.m_identifier,"|");
	    while (st.hasMoreTokens()){
		try{
		    String orgurl=st.nextToken();
		    url=new URL(orgurl);
		    filename=	r.m_id.replace('/','.');
		    filename=filename.replace('\\','.');
		    filename=filename.replace(':','.');
		    filename=filename.replace(' ','.');
		    filename=filename+orgurl.substring(orgurl.lastIndexOf("."));
		    break;
		}
		catch(Exception ex){		  
		    		
		}
		return false;
	    }

	    File file=new File(cachecfg.dir+File.separatorChar+filename);
	    
	    FileOutputStream bufferOut=new FileOutputStream(file);
	   
	    byte[] buffer=new byte[10000];
	    InputStream in =url.openStream();
	    BufferedInputStream bufferIn=new BufferedInputStream(in);
	    int len=0;
	    while((len= bufferIn.read(buffer,0,10000))!=-1)  {
		bufferOut.write(buffer,0,len);
	    }
	    
	    bufferOut.close();
	    result=true;
	}
	
        catch (Exception e){
            System.out.println(e);
            result=false;
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }
        return result;

    }
    
    private void delete(Record r){
        Statement s=null;
        try{
	    
            s=connection.createStatement();
	    
	    
	    try{
		idexist=s.executeUpdate("delete from dc where id='"+clean(r.m_id.trim())+"'");
	    }catch(Exception ex){
	    }
	    
	    try{
                        idexist=s.executeUpdate("delete from free where id='"+clean(r.m_id.trim())+"'");
	    }catch(Exception ex){
	    }
	    
            
	    
        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }


    }
    /**update Partition*/
    public void updatePartition(Record r){
        Statement s=null;
        try{
            s=connection.createStatement();
            ResultSet rset= s.executeQuery("select sets from dc where id='"+r.m_id.trim()+"'");
            rset.next();
            String oldpartition=rset.getString(1);
	    
            if (oldpartition==null){
		
                String command="update dc set sets='"+r.m_sets+"' where id='"+r.m_id.trim()+"'";
                s.executeUpdate("update dc set sets='"+r.m_sets+"' where id='"+r.m_id.trim()+"'");
            }
            else  if (oldpartition.indexOf(r.m_sets)==-1){
                s.executeUpdate("update dc set sets='"+oldpartition+" | "+r.m_sets+"' where id='"+r.m_id.trim()+"'");
            }
        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                s.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }
	
    }
    
   
    
    /** update Partition*/
    public void updatePartition(RecordList list){
	for (int i=0;i<list.size();i++)
	    updatePartition((Record)list.get(i));
    }
 
    private void writeUpdate(Record r, String olddatestamp){
        Statement stmt=null;
	String sql = null;
        try{
            stmt=connection.createStatement();
	    try{
		sql="delete from updaterecord where id='"+clean(r.m_id.trim())+"'";
		stmt.executeUpdate(sql);
		sql = "insert into updaterecord values ('"+clean(r.m_id.trim())+"','"+clean(r.m_archive)+"', '"+clean(r.m_datestamp)+"', '"+olddatestamp+"', null)";
		stmt.executeUpdate(sql);
            }
            catch(Exception ex){
            }
	    
        }
        catch (SQLException e){
            System.out.println(e);
        }
        finally{
            try{
                stmt.close();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }

    }
    



}








