package scheduler;


import java.util.*;
import oairequester.*;
import log.*;
/** Harvest by partition */
public abstract class HarvestByPartition implements Task{
    DataProvider dp;
    Record2DB oo;
    int interval;
    
    Enumeration enum;
    public final String FAKEFLAG="************************";
    boolean done=false;
     
    MetaFormatsList mfList=null;
    LogRecord m_logrecord = null;
    Log m_log;
  
    String granularity="yyyy-MM-dd";
    /** 
     * Constructor
     * @param dp the data provider
     * @param oo
     * @param m_log
     */

    public  HarvestByPartition(DataProvider dp,Record2DB oo, Log m_log){
        this.dp=dp;
        this.oo=oo;
	this.m_log = m_log;
    }

    /** 
     * prepare.
     */

    public void prepare() throws oairequester.ParseException,java.io.IOException{
	 m_logrecord = new LogRecord(Calendar.getInstance().getTime(), "Identify",dp.getId(),"Request",false);
	 m_log.writeBeginLog(m_logrecord);
         done=false;
         dp.identify.read();
         IdentifyInfo info=(IdentifyInfo)dp.identify.parse ();
         oo.updateIdentify(info);

         if (info.granularity !=null)
           granularity = info.granularity;

	 m_logrecord.m_status = "OK";
	 m_logrecord.m_detail = dp.getUrl() + "?verb=Identify";
	 m_logrecord.m_totalnum = 1;
	 m_log.writeEndLog(m_logrecord, dp.getId());

         /** If there is no specification on Sets, then harvest without using partition
             Otherwise, harvest by customised partition list*/
         Vector pList = new Vector();
    
         m_logrecord = new LogRecord(Calendar.getInstance().getTime(), "ListSets",dp.getId(),"Request",false);
         m_log.writeBeginLog(m_logrecord);

         dp.listsets.read();
         PartitionList pl=(PartitionList)dp.listsets.parse();
  
	 m_logrecord.m_status = "OK";
         m_logrecord.m_detail = dp.getUrl() + "?verb=ListSets";
         m_logrecord.m_totalnum = pList.size();
         m_log.writeEndLog(m_logrecord, dp.getId()); 

         if(dp.getSet()==null || dp.getSet().trim().length()==0 || dp.getSet().trim().equalsIgnoreCase("null"))
	     pList.add(FAKEFLAG);
         
         else if (dp.getSet().trim().equalsIgnoreCase("all")){ //Harvest all sets

            pList=pl.getHierarchyPartition(1);
            if (pList.size()==0)
               pList.add(FAKEFLAG);
           
	 }
         else { //Harvest specified sets only
	     StringTokenizer st = new StringTokenizer( dp.getSet().trim(), " ,;|" );
             while (st.hasMoreTokens())
		 pList.add(st.nextToken());
	 }
         enum=pList.elements();
        
         /** By default, Harvest all kinds of metadata formats.
             Otherwise, harvest by customised formats */
	 mfList= new MetaFormatsList();
         String mFmt = dp.getMetaFormat();
         if(mFmt == null || mFmt.equalsIgnoreCase("all") || mFmt.trim().equals("") || mFmt.trim().equalsIgnoreCase("null") ){ // All metadata formats    
           String id =null;
           dp.listMetaFormats.read(id);
           mfList = (MetaFormatsList)dp.listMetaFormats.parse();
         }
         else{ // Only customised formats
             StringTokenizer st = new StringTokenizer( mFmt, " ,;|" );
             while (st.hasMoreTokens())
	       mfList.add(st.nextToken()); 
	 }

    }
    /**
     * @return true of done, false otherwise.
     */
    public boolean isDone(){
        return done;
    }
    /**
     * @return log record
     */
    public LogRecord getLog() {
	return m_logrecord;
    }

}





