package search;
// class DBSearcher

// version 1.3 -- April 20, 2000 by Dun Tan
// Only provide the partial result set and only do search when needed
// catch SQLException internally when doing the search
// fix the index out of bound problem

import java.sql.*;
import java.io.*;
import java.util.*;

public class DBSearcher 
{
    int maxResultSize ;
    public SearchParam sp;
    public static SearchParam sp2;
    private int numOfHits;    // the total number of hits
    private int numFilled;    // the number of hits contained in the current result set
    private int resultIndex;  // the absolute index into the whole result set
    private int nStart;       // the first record in the current result set
    private int nEnd;         // the last record in the current result set
    private ArrayList idCollection; // with each element to be a two element array:
				    // String id and String tag specifying table name
    private Hashtable tb;     // the number of hits in each table
    private String query;
    public GroupItem groupItem;
////////////////////////////////////////////////////////////////////
    /**
     * @param params user input
     * @param groupitem group field, only support archive name now
     */

    public DBSearcher(SearchParam params,GroupItem item) 
    {
    	sp=params;
    	sp2 = sp;
	numOfHits = 0;
    	numFilled = 0;
	resultIndex = 0;
        maxResultSize =250;
	nStart = 0;
    	nEnd = 0;
    	idCollection = new ArrayList(maxResultSize);
        this.groupItem=item;
        query=formQueryStr();
        System.out.println(query);

    }
////////////////////////////////////////////////////////////////////
    private String formQueryStr()
    {
        String tmp=sp.getWhereClause()==null?"":sp.getWhereClause();

        if (tmp.indexOf("where")==-1){
            tmp=tmp+" where "+sp.getGroupClause()+"='"+groupItem.groupid+"' ";
        }

        /* There is serious error here!!!!!!!!!!!!!!!, for the time being, I did not implement it
         * if user happends input something similar to groupClause, system can not do the right group
         * current used groupclause is "archive"
         */

        else if (tmp.indexOf(sp.getGroupClause ())==-1){
            tmp=tmp+" and "+ sp.getGroupClause()+" ='"+groupItem.groupid+"' ";
        }
        if (sp.getSortClause()!=null)
            tmp=tmp+sp.getSortClause();

        return tmp;
    }
/////////////////////////////////////////////////////////

    // Build CMetaData object given the row data from oracle
    // Input  : rs -- ResultSet of executed SQL query
    // Output : CMetaData
    private static CMetaData buildMDObject(ResultSet rs) throws SQLException
    {
    	CMetaData md = new CMetaData();

		md.id = rs.getString(1);
	    md.archive = rs.getString(2);
        md.datestamp =rs.getString(3);
        md.sets=rs.getString(4);
        md.title = rs.getString(5);
        md.mark_title = sp2.mark_title;
    	md.creator = rs.getString(6);
    	md.mark_creator = sp2.mark_creator;
       	md.subject = rs.getString(7);
	    md.setDescription(rs.getString(8));
	    md.mark_description = sp2.mark_description;
       	md.publisher = rs.getString(9);
        md.contributor = rs.getString(10);
        md.discovery = rs.getString(11);
        md.type = rs.getString(12);
        md.format = rs.getString(13);
        md.identifier=rs.getString(14);
        md.source=rs.getString(15);
        md.language=rs.getString(16);
        md.relation=rs.getString(17);
        md.coverage=rs.getString(18);
        md.rights=rs.getString(19);
        md.cache=rs.getString(20);
		md.normDiscovery = rs.getString(21);
        md.status=rs.getString(22);
       	return md;
    }

    /**
     * Search Database
     * @param con -- established JDBC connection
     */

    public void DoSearch(Connection con) throws IOException {
	 

    	Statement stmt = null;
	    try {
	        stmt = con.createStatement();
    	    System.out.println(query);
            numOfHits = groupItem.size;
            System.out.println("search.DBSearcher.DoSearch numOfHits = " + numOfHits + "stmt = " + stmt + " tName = " + sp.getTableName() + " query = " + query);
    	    SearchAndFill(stmt, sp.getTableName(), query, 1);
    	    stmt.close();

	    } catch (SQLException e) {
	        System.out.println(e);
	        System.out.println("search.DBSearcher.DoSearch Problem doing search");
        }
    }

    // Fill the id collections for a specified archive
    // Input  : rs -- object of SQL ResultSet
    //          tag -- Indicate which table the id is from
    //          nst -- the absolute index of the start record
    // Output : None
    private void fill(ResultSet rs, String tag, int nst) throws SQLException, IOException {


	while (rs.next() && numFilled < maxResultSize) {
	    resultIndex ++;

	    if (resultIndex >= nst) {
		idCollection.add(rs.getString(1));

		numFilled ++;
	    }
	}

    }



    // Search and provide the result in the given set
    // Input  : stmt -- object of SQl Statement
    //          tName -- the name of table to be searched
    //          q -- the SQL query to be executed
    //          setNum -- the number of the result set required
    // Output : None
    private void SearchAndFill(Statement stmt, String tName, String q, int setNum) throws SQLException, IOException {
	nStart = 1 + (setNum - 1) * maxResultSize;
   

	ResultSet rset = stmt.executeQuery("select id from "+tName+" "+q);

        System.out.println("search.DBSearcher.SearchAndFill qry=select id from "+tName+" "+q);

	fill(rset, tName, nStart);
	rset.close();

	nEnd = nStart + numFilled - 1;
	
    }


    // Return the ids collection
    // Input  : None
    // Output : ArrayList
    public ArrayList GetIdCollection () {
	return idCollection;
    }

    // Convert the absolute index to the number of block
    // Input  : int -- the absolute index of the record
    // Output : int -- the block that contains the given record
    private int indexToSetNum(int index) {
	double d = (double)index/(double)maxResultSize;
	int setNum = (int)Math.ceil(d);
	return setNum;
    }

    // Return the meta data for a given index of hits, using the specified connection
    // Input  : con -- established JDBC connection
    //          index -- index of the meta data returned, start from zero
    public CMetaData getMetaData(Connection con, int index) throws IOException 
    {
	CMetaData md = new CMetaData();
	int idSize = idCollection.size();
	ResultSet rs;
	int index2Id;

	Statement stmt = null;

	try {
	    stmt = con.createStatement();

	    index2Id = index + 1;

	    if (idSize < numOfHits && (index2Id > nEnd || index2Id < nStart)) { // redo the search
		int setNum = indexToSetNum(index2Id);
		index2Id = index2Id % maxResultSize;

		// reset the parameter
		numFilled = 0;
		resultIndex = 0;
		idCollection.clear();

	    SearchAndFill(stmt, sp.getTableName(), query, setNum);
	    }


	    String id = (String)idCollection.get(index % maxResultSize);
        md=getMetaData(con,id);

        return md;

	} catch (SQLException e0) {
            System.out.println("Problem getting MetaData");
	    try {
		if (stmt != null)
		    stmt.close();
	    } catch (SQLException e3) {}

	    return null;
	}
    }
//////////////////////////////////////////////////////////////////

    public static  CMetaData getMetaData(Connection con,String id){
        CMetaData md=null;
    	ResultSet rs;
      	Statement stmt = null;
	
   	    try {
            stmt = con.createStatement();
            rs = stmt.executeQuery("select dc.id,dc.archive,dc.datestamp,dc.sets,dc.title,dc.creator,dc.subject,dc.description,dc.publisher,dc.contributor,dc.discovery,dc.type,dc.format,dc.identifier,dc.source,dc.language,dc.relation,dc.coverage,dc.rights,dc.cache,dc.normdiscovery,dc.status from dc where id= '"+id+"'");

    	    rs.next();
	        md = buildMDObject(rs);

	        rs.close();
    	    stmt.close();
        } catch (SQLException e1) 
        {
            try {
		            if (stmt != null)
    		    	    stmt.close();
            } catch (SQLException e2)
            {
            }

		  return null;
        }

	    return md;

    }
//////////////////////////////////////////////////////////////////
	public  int getIndex (String id)
	{
		int loc = idCollection.indexOf (id);
		if (loc<0)
			return loc;
		return loc + GetStart ()-1;	
	}
//////////////////////////////////////////////////////////////////
    // Return the index of the first record of the current set in terms of the whole search result set
    // Input  : None
    // Output : int
    public int GetStart() 
    {
		return nStart;
    }

    // Return the index of the last record of the current set in terms of the whole search result set
    // Input  : None
    // Output : int
    public int GetEnd() 
    {
		return nEnd;
    }

    // After call DoSearch(), call this function, it will return the number of hits
    // Input  : None
    // Output : int -- the number of search hits
    public int GetNumHits() 
    {
		return numOfHits;
    }

};






